@@ -11,6 +11,7 @@
 #include <librsvg/rsvg-cairo.h>
 #endif
 
+#include "eog-config-keys.h"
 #include "eog-marshal.h"
 #include "eog-scroll-view.h"
 #include "eog-debug.h"
@@ -72,6 +73,14 @@ static GtkTargetEntry target_table[] = {
 	{ "text/uri-list", 0, 0},
 };
 
+enum {
+	PROP_0,
+	PROP_BACKGROUND_COLOR,
+	PROP_SCROLLWHEEL_ZOOM,
+	PROP_USE_BG_COLOR,
+	PROP_ZOOM_MULTIPLIER
+};
+
 /* Private part of the EogScrollView structure */
 struct _EogScrollViewPrivate {
 	/* some widgets we rely on */
@@ -138,6 +147,10 @@ struct _EogScrollViewPrivate {
 	/* the type of the cursor we are currently showing */
 	EogScrollViewCursor cursor;
 
+	gboolean  use_bg_color;
+	GdkColor *background_color;
+	GdkColor *override_bg_color;
+
 	cairo_surface_t *background_surface;
 };
 
@@ -1950,6 +1963,11 @@ eog_scroll_view_set_transparency (EogScrollView *view, EogTransparencyStyle styl
 	}
 
 	if (changed && priv->pixbuf != NULL && gdk_pixbuf_get_has_alpha (priv->pixbuf)) {
+		if (priv->background_surface) {
+			cairo_surface_destroy (priv->background_surface);
+			/* Will be recreated if needed during redraw */
+			priv->background_surface = NULL;
+		}
 		gtk_widget_queue_draw (GTK_WIDGET (priv->display));
 	}
 }
@@ -2168,13 +2186,55 @@ eog_scroll_view_scrollbars_visible (EogScrollView *view)
     object creation/freeing
   ---------------------------------*/
 
+static gboolean
+sv_string_to_color_mapping (GValue   *value,
+			    GVariant *variant,
+			    gpointer  user_data)
+{
+	GdkColor color;
+
+	g_return_val_if_fail (g_variant_is_of_type (variant, G_VARIANT_TYPE_STRING), FALSE);
+
+	if (gdk_color_parse (g_variant_get_string (variant, NULL), &color)) {
+		g_value_set_boxed (value, &color);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static GVariant*
+sv_color_to_string_mapping (const GValue       *value,
+			    const GVariantType *expected_type,
+			    gpointer            user_data)
+{
+	GVariant *variant = NULL;
+	GdkColor *color;
+	gchar *hex_val;
+
+	g_return_val_if_fail (G_VALUE_TYPE (value) == GDK_TYPE_COLOR, NULL);
+	g_return_val_if_fail (g_variant_type_equal (expected_type, G_VARIANT_TYPE_STRING), NULL);
+
+	color = g_value_get_boxed (value);
+	hex_val = g_strdup_printf ("#%02X%02X%02X",
+				   color->red / 256,
+				   color->green / 256,
+				   color->blue / 256);
+	variant = g_variant_new_string (hex_val);
+	g_free (hex_val);
+
+	return variant;
+}
+
 
 static void
 eog_scroll_view_init (EogScrollView *view)
 {
+	GSettings *settings;
 	EogScrollViewPrivate *priv;
 
 	priv = view->priv = EOG_SCROLL_VIEW_GET_PRIVATE (view);
+	settings = g_settings_new (EOG_CONF_VIEW);
 
 	priv->zoom = 1.0;
 	priv->min_zoom = MIN_ZOOM_FACTOR;
@@ -2192,6 +2252,17 @@ eog_scroll_view_init (EogScrollView *view)
 	priv->transp_color = 0;
 	priv->cursor = EOG_SCROLL_VIEW_CURSOR_NORMAL;
 	priv->menu = NULL;
+	
+	g_settings_bind (settings, EOG_CONF_VIEW_USE_BG_COLOR, view,
+			 "use-background-color", G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind_with_mapping (settings, EOG_CONF_VIEW_BACKGROUND_COLOR,
+				      view, "background-color",
+				      G_SETTINGS_BIND_DEFAULT,
+				      sv_string_to_color_mapping,
+				      sv_color_to_string_mapping, NULL, NULL);
+	g_object_unref (settings);
+
+	priv->override_bg_color = NULL;
 	priv->background_surface = NULL;
 }
 
@@ -2216,6 +2287,16 @@ eog_scroll_view_dispose (GObject *object)
 		priv->idle_id = 0;
 	}
 
+	if (priv->background_color != NULL) {
+		gdk_color_free (priv->background_color);
+		priv->background_color = NULL;
+	}
+
+	if (priv->override_bg_color != NULL) {
+		gdk_color_free (priv->override_bg_color);
+		priv->override_bg_color = NULL;
+	}
+
 	if (priv->background_surface != NULL) {
 		cairo_surface_destroy (priv->background_surface);
 		priv->background_surface = NULL;
@@ -2226,6 +2307,71 @@ eog_scroll_view_dispose (GObject *object)
 	G_OBJECT_CLASS (eog_scroll_view_parent_class)->dispose (object);
 }
 
+static void
+eog_scroll_view_get_property (GObject *object, guint property_id,
+			      GValue *value, GParamSpec *pspec)
+{
+	EogScrollView *view;
+	EogScrollViewPrivate *priv;
+
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (object));
+
+	view = EOG_SCROLL_VIEW (object);
+	priv = view->priv;
+
+	switch (property_id) {
+	case PROP_USE_BG_COLOR:
+		g_value_set_boolean (value, priv->use_bg_color);
+		break;
+	case PROP_BACKGROUND_COLOR:
+		//FIXME: This doesn't really handle the NULL color.
+		g_value_set_boxed (value, priv->background_color);
+		break;
+	case PROP_SCROLLWHEEL_ZOOM:
+		g_value_set_boolean (value, priv->scroll_wheel_zoom);
+		break;
+	case PROP_ZOOM_MULTIPLIER:
+		g_value_set_double (value, priv->zoom_multiplier);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	}
+}
+
+static void
+eog_scroll_view_set_property (GObject *object, guint property_id,
+			      const GValue *value, GParamSpec *pspec)
+{
+	EogScrollView *view;
+	EogScrollViewPrivate *priv;
+
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (object));
+
+	view = EOG_SCROLL_VIEW (object);
+	priv = view->priv;
+
+	switch (property_id) {
+	case PROP_USE_BG_COLOR:
+		eog_scroll_view_set_use_bg_color (view, g_value_get_boolean (value));
+		break;
+	case PROP_BACKGROUND_COLOR:
+	{
+		const GdkColor *color = g_value_get_boxed (value);
+		eog_scroll_view_set_background_color (view, color);
+		break;
+	}
+	case PROP_SCROLLWHEEL_ZOOM:
+		eog_scroll_view_set_scroll_wheel_zoom (view, g_value_get_boolean (value));
+		break;
+	case PROP_ZOOM_MULTIPLIER:
+		eog_scroll_view_set_zoom_multiplier (view, g_value_get_double (value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	}
+}
+
+
 static void
 eog_scroll_view_class_init (EogScrollViewClass *klass)
 {
@@ -2236,6 +2382,49 @@ eog_scroll_view_class_init (EogScrollViewClass *klass)
 	widget_class = (GtkWidgetClass*) klass;
 
 	gobject_class->dispose = eog_scroll_view_dispose;
+        gobject_class->set_property = eog_scroll_view_set_property;
+        gobject_class->get_property = eog_scroll_view_get_property;
+
+	/**
+	 * EogScrollView:background-color:
+	 *
+	 * This is the default background color used for painting the background
+	 * of the image view. If set to %NULL the color is determined by the
+	 * active GTK theme.
+	 */
+	g_object_class_install_property (
+		gobject_class, PROP_BACKGROUND_COLOR,
+		g_param_spec_boxed ("background-color", NULL, NULL,
+				    GDK_TYPE_COLOR,
+				    G_PARAM_READWRITE | G_PARAM_STATIC_NAME));
+
+	g_object_class_install_property (
+		gobject_class, PROP_USE_BG_COLOR,
+		g_param_spec_boolean ("use-background-color", NULL, NULL, FALSE,
+				      G_PARAM_READWRITE | G_PARAM_STATIC_NAME));
+
+	/**
+	 * EogScrollView:zoom-multiplier:
+	 *
+	 * The current zoom factor is multiplied with this value + 1.0 when
+	 * scrolling with the scrollwheel to determine the next zoom factor.
+	 */
+	g_object_class_install_property (
+		gobject_class, PROP_ZOOM_MULTIPLIER,
+		g_param_spec_double ("zoom-multiplier", NULL, NULL,
+				     -G_MAXDOUBLE, G_MAXDOUBLE - 1.0, 0.05,
+				     G_PARAM_READWRITE | G_PARAM_STATIC_NAME));
+
+	/**
+	 * EogScrollView:scrollwheel-zoom:
+	 *
+	 * If %TRUE the scrollwheel will zoom the view, otherwise it will be
+	 * used for scrolling a zoomed image.
+	 */
+	g_object_class_install_property (
+		gobject_class, PROP_SCROLLWHEEL_ZOOM,
+		g_param_spec_boolean ("scrollwheel-zoom", NULL, NULL, TRUE,
+				      G_PARAM_READWRITE | G_PARAM_STATIC_NAME));
 
 	view_signals [SIGNAL_ZOOM_CHANGED] =
 		g_signal_new ("zoom_changed",
@@ -2438,13 +2627,93 @@ eog_scroll_view_set_popup (EogScrollView *view,
 			  G_CALLBACK (view_on_button_press_event_cb), NULL);
 }
 
+static gboolean
+_eog_gdk_color_equal0 (const GdkColor *a, const GdkColor *b)
+{
+	if (a == NULL || b == NULL)
+		return (a == b);
+
+	return gdk_color_equal (a, b);
+}
+
+static gboolean
+_eog_replace_gdk_color (GdkColor **dest, const GdkColor *new)
+{
+	GdkColor *old = *dest;
+
+	if (_eog_gdk_color_equal0 (old, new))
+		return FALSE;
+
+	if (old != NULL)
+		gdk_color_free (old);
+
+	*dest = (new) ? gdk_color_copy (new) : NULL;
+
+	return TRUE;
+}
+
+static void
+_eog_scroll_view_update_bg_color (EogScrollView *view)
+{
+	const GdkColor *selected;
+	EogScrollViewPrivate *priv = view->priv;
+
+	if (!priv->use_bg_color)
+		selected = NULL;
+	else if (priv->override_bg_color)
+		selected = priv->override_bg_color;
+	else
+		selected = priv->background_color;
+
+	if (priv->transp_style == EOG_TRANSP_BACKGROUND
+	    && priv->background_surface != NULL) {
+		/* Delete the SVG background to have it recreated with
+		 * the correct color during the next SVG redraw */
+		cairo_surface_destroy (priv->background_surface);
+		priv->background_surface = NULL;
+	}
+
+	gtk_widget_modify_bg (GTK_WIDGET (view),
+			      GTK_STATE_NORMAL,
+			      selected);
+}
+
 void
-eog_scroll_view_set_bg_color (EogScrollView *view,
-			      GdkColor *color)
+eog_scroll_view_set_background_color (EogScrollView *view,
+				      const GdkColor *color)
 {
 	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
 
-	gtk_widget_modify_bg (GTK_WIDGET (view), GTK_STATE_NORMAL, color);
+	if (_eog_replace_gdk_color (&view->priv->background_color, color))
+		_eog_scroll_view_update_bg_color (view);
+}
+
+void
+eog_scroll_view_override_bg_color (EogScrollView *view,
+				   const GdkColor *color)
+{
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
+
+	if (_eog_replace_gdk_color (&view->priv->override_bg_color, color))
+		_eog_scroll_view_update_bg_color (view);
+}
+
+void
+eog_scroll_view_set_use_bg_color (EogScrollView *view, gboolean use)
+{
+	EogScrollViewPrivate *priv;
+
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
+
+	priv = view->priv;
+
+	if (use != priv->use_bg_color) {
+		priv->use_bg_color = use;
+
+		_eog_scroll_view_update_bg_color (view);
+
+		g_object_notify (G_OBJECT (view), "use-background-color");
+	}
 }
 
 void
@@ -2453,7 +2722,10 @@ eog_scroll_view_set_scroll_wheel_zoom (EogScrollView *view,
 {
 	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
 
-        view->priv->scroll_wheel_zoom = scroll_wheel_zoom;
+	if (view->priv->scroll_wheel_zoom != scroll_wheel_zoom) {
+	        view->priv->scroll_wheel_zoom = scroll_wheel_zoom;
+		g_object_notify (G_OBJECT (view), "scrollwheel-zoom");
+	}
 }
 
 void
@@ -2463,4 +2735,6 @@ eog_scroll_view_set_zoom_multiplier (EogScrollView *view,
 	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
 
         view->priv->zoom_multiplier = 1.0 + zoom_multiplier;
+
+	g_object_notify (G_OBJECT (view), "zoom-multiplier");
 }
