@@ -50,8 +50,11 @@ void     eog_scroll_view_set_antialiasing_out (EogScrollView *view, gboolean sta
 void     eog_scroll_view_set_transparency (EogScrollView *view, EogTransparencyStyle style, GdkColor *color);
 gboolean eog_scroll_view_scrollbars_visible (EogScrollView *view);
 void	 eog_scroll_view_set_popup (EogScrollView *view, GtkMenu *menu);
-void	 eog_scroll_view_set_bg_color (EogScrollView *view, GdkColor *color);
-
+void	 eog_scroll_view_set_background_color (EogScrollView *view,
+					       const GdkColor *color);
+void	 eog_scroll_view_override_bg_color (EogScrollView *view,
+					    const GdkColor *color);
+void     eog_scroll_view_set_use_bg_color (EogScrollView *view, gboolean use);
 /* zoom api */
 void     eog_scroll_view_zoom_in          (EogScrollView *view, gboolean smooth);
 void     eog_scroll_view_zoom_out         (EogScrollView *view, gboolean smooth);
